#include <linux/init.h>
#include <linux/module.h>
#include <asm/uaccess.h>
#include <linux/fs.h>
	
#include <linux/device.h>
#include <linux/cdev.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/pm.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/nodemask.h>
#include <linux/clk.h>
#include <linux/platform_device.h>
#include <linux/fsl_devices.h>
#include <linux/ipu.h>
#include <linux/mxcfb.h>
#include <linux/fec.h>
#include <linux/memblock.h>
#include <linux/gpio.h>

//UART5 GPIO_7_10
#define IMX6Q_GPIO485_EN01_BASE_ADDR 0x020b4000
#define IMX6Q_GPIO485_EN01_DR_BASE_ADDR 0x00
#define IMX6Q_GPIO485_EN01_GDIR_BASE_ADDR 0x04

//UART3 GPI0_3_32
#define IMX6Q_GPIO485_EN02_BASE_ADD 0x020a4000
#define IMX6Q_GPIO485_EN02_DR_BASE_ADDR 0x00
#define IMX6Q_GPIO485_EN02_GDIR_BASE_ADDR 0x04


int major_RS485;
int RS485_ret;


struct RS485_device_data{
	unsigned int led_major;
	struct cdev 	*RS485_cdev;
	struct device 	*RS485_device;
	struct class 	*RS485_class;
	dev_t			devno;
	void __iomem	*RS485_base01;
	void __iomem	*RS485_base02;
};

static struct RS485_device_data *RS485_DEV;

 ssize_t write_RS485(struct file *file, const char __user *buf, size_t size, loff_t *loff) 	
{
	//printk(KERN_ERR"rs485_weite");
	
	 int val;
	 val = *buf;
	//printk(KERN_ERR"%d\n",val);
	switch (val)
	{
		case 1:
		//	printk(KERN_ERR"%d\n",val);
		//	printk(KERN_ERR"1 on ");
			writel(readl(RS485_DEV->RS485_base02+0x00) |(0x01<< 23),RS485_DEV->RS485_base02+0x00);
			break;
		case 2:	
		//	printk(KERN_ERR"%d\n",val);
		//	printk(KERN_ERR"1 off ");
			writel(readl(RS485_DEV->RS485_base02+0x00) &~(0x01<< 23),RS485_DEV->RS485_base02+0x00);
			break;	
			break;
		case 3:
		//	printk(KERN_ERR"%d\n",val);
		//	printk(KERN_ERR"2 on ");
			writel(readl(RS485_DEV->RS485_base01+0x00) | (0x01<< 10),RS485_DEV->RS485_base01+0x00);
			break;
		case 4:
		//	printk(KERN_ERR"%d\n",val);
		//	printk(KERN_ERR"2 off ");
			writel(readl(RS485_DEV->RS485_base01+0x00) &~(0x01<< 10) ,RS485_DEV->RS485_base01+0x00);	
		default:
			break;
	}
	return 0;

}
int open_RS485 (struct file *file, const char __user *buf, size_t size, loff_t *loff)

{
	//printk(KERN_ERR"rs485_open");

	writel(readl(RS485_DEV->RS485_base01+0x04) |(0x01<< 10) ,RS485_DEV->RS485_base01+0x04);
	writel(readl(RS485_DEV->RS485_base02+0x04) |(0x01<< 23) ,RS485_DEV->RS485_base02+0x04);

	
	
	return 0;
}
struct file_operations RS485_ops = {
	.owner = THIS_MODULE,
	.open = open_RS485,
	.write = write_RS485,
};
static int __init  RS485_init(void)
{	
	//printk(KERN_ERR"rs485_init");
	int ret;
	RS485_DEV=kmalloc(sizeof(struct RS485_device_data), GFP_KERNEL);
	if(RS485_DEV==NULL){
		printk(KERN_ERR "no memory for malloc!\n");
		return -ENOMEM;
	}

	ret=alloc_chrdev_region(&RS485_DEV->devno,0,1,"RS485_MODULE");
	if(ret<0){
		printk(KERN_ERR "register major failed!\n");
		ret =-EINVAL;
		goto output0;
	}

	RS485_DEV->RS485_cdev =cdev_alloc();
	if(RS485_DEV->RS485_cdev==NULL){
		printk(KERN_ERR "alloc memory failed\n");
		ret =-ENOMEM;
		goto output1;
	}
	cdev_init(RS485_DEV->RS485_cdev, &RS485_ops);
	cdev_add(RS485_DEV->RS485_cdev,RS485_DEV->devno,1);

	RS485_DEV->RS485_class=class_create(THIS_MODULE, "RS485_class");
	if(IS_ERR(RS485_DEV->RS485_class)){
		printk(KERN_ERR "create class failed!\n");
		ret =PTR_ERR(RS485_DEV->RS485_class);
		goto output2;
	}
	RS485_DEV->RS485_device=device_create(RS485_DEV->RS485_class,NULL,RS485_DEV->devno,NULL,"RS485");
	if(IS_ERR(RS485_DEV->RS485_device)){
		printk(KERN_ERR "create device failed!\n");
		ret =PTR_ERR(RS485_DEV->RS485_device);
		goto output3;
	}

	RS485_DEV->RS485_base01 = ioremap(IMX6Q_GPIO485_EN01_BASE_ADDR,8);
	RS485_DEV->RS485_base02 = ioremap(IMX6Q_GPIO485_EN02_BASE_ADD,8);


	
return 0;
	
output3:
	class_destroy(RS485_DEV->RS485_class);
output2:
	cdev_del(RS485_DEV->RS485_cdev);
output1:
	unregister_chrdev_region(RS485_DEV->devno,1);
output0:
	kfree(RS485_DEV);
	return ret;
	

}

static void __exit RS485_exit(void)
{
	unregister_chrdev_region(RS485_DEV->devno,1);
	cdev_del(RS485_DEV->RS485_cdev);
	device_destroy(RS485_DEV->RS485_class,RS485_DEV->devno);
	class_destroy(RS485_DEV->RS485_class);
	iounmap(RS485_DEV->RS485_base01);
	iounmap(RS485_DEV->RS485_base02);
	kfree(RS485_DEV);
}

module_init(RS485_init);
module_exit(RS485_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("www.waveshare.com");
MODULE_DESCRIPTION("driver for RS485");



