#include <linux/module.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/mutex.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/stat.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/tty.h>
#include <linux/kmod.h>
#include <linux/gfp.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/pm.h>
#include <linux/slab.h>
#include <linux/sysctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <linux/poll.h> 

#define IMX_GPIO_NR(bank, nr)		(((bank) - 1) * 32 + (nr))

static char data[2];

#define DEVICE_NAME 	"ds18b20"
#define _debug(fmt, arg...)	printk(KERN_ERR fmt, ##arg)
typedef struct __GPIO_DESC {
	int gpio;
	const char *desc;
}gpio_desc;
static gpio_desc led_dev_data[] = {
	{IMX_GPIO_NR(7, 13),"ds18b20"},	/* port 13 for ds18b20 */
};


void DS18b20_write_byte (char byte)
{
    char i;
    gpio_direction_output(led_dev_data[0].gpio, 0);	
    for (i = 0; i < 8; i++)
    {
    	gpio_set_value(led_dev_data[0].gpio, 0);
		udelay(1); 
		
        if(byte & 0x01)
        {
        	gpio_set_value(led_dev_data[0].gpio, 1);
        }
        else 
        {
			;
        }
   
        udelay(60);

        gpio_set_value(led_dev_data[0].gpio, 1);
		
        udelay(15);

        byte >>= 1;
    }
    gpio_set_value(led_dev_data[0].gpio, 1);
} 


char DS18b20_read_byte (void)
{
    char i = 0;
    char byte = 0;
    for (i = 0; i < 8; i++)
    {
        gpio_direction_output(led_dev_data[0].gpio, 0);	
        gpio_set_value(led_dev_data[0].gpio, 0);

        udelay(1);
        byte >>= 1;

        gpio_set_value(led_dev_data[0].gpio, 1);
        
    	gpio_direction_input(led_dev_data[0].gpio);

        if (gpio_get_value(led_dev_data[0].gpio))
   			byte |= 0x80;
        udelay(60);
    }
    return byte;       
}



/*reset 18b20	*/	
char ds18b20_reset (void)
{
	int try_cnt = 100;
	int flag = 0;
	while(try_cnt--)
	{
		gpio_direction_output(led_dev_data[0].gpio, 0);		
		
		gpio_set_value(led_dev_data[0].gpio, 1);
		udelay(100);

		gpio_set_value(led_dev_data[0].gpio, 0);
		udelay(600);
		
		gpio_set_value(led_dev_data[0].gpio, 1);
		udelay(100);
		
		gpio_direction_input(led_dev_data[0].gpio);
		
	
		if(!gpio_get_value(led_dev_data[0].gpio))
		{ 
			//succeed reset
			flag = 1;
			break;
		}
	}
    
	if(flag)
	{
		//_debug("succeed reset the sensor\n");
		return 1;
	}
	else
	{
		//_debug("reset the sensor failed\n");
		return 0;
	}

} 


void DS18b20_proc(void)         
{
    ds18b20_reset();
    
    udelay(120);
    
    DS18b20_write_byte(0xcc);
    DS18b20_write_byte(0x44);
    
    udelay(5);
    
    ds18b20_reset();
    udelay(200);
    
    DS18b20_write_byte(0xcc);
    DS18b20_write_byte(0xbe);
    
    data[0] = DS18b20_read_byte();
    data[1] = DS18b20_read_byte();
	
} 

static int ds18B20_open(struct inode *inode, struct file *file)
{
	int ret = -1;
	
	
	ret = gpio_request(led_dev_data[0].gpio,led_dev_data[0].desc);
	if(ret < 0)
	{
		_debug("failed to request GPIO %d{%s}, error %d\n",	led_dev_data[0].gpio,led_dev_data[0].desc, ret);
		return ret;
	}

	 	
	ds18b20_reset(); 
	return 0;

}


static ssize_t ds18b20_read(struct file *filp, char *buf, size_t len, loff_t *off)
{
	
    DS18b20_proc();

    buf[0] = data[0];
    buf[1] = data[1];
    return 1;
}



static int ds18b20_close(struct inode *inode, struct file *file)
{
	
	printk("Close device sucessfully\n");
	
	gpio_free(led_dev_data[0].gpio);
	
	return 0;
}


static struct file_operations ds18b20_fops=
{
    .owner			=	THIS_MODULE,
	.open 			= 	ds18B20_open,
	.read			=   ds18b20_read,
	.release	 	= 	ds18b20_close,
};

static struct miscdevice misc = {
	.minor	= MISC_DYNAMIC_MINOR,
	.name	= DEVICE_NAME,
	.fops	= &ds18b20_fops,
};


static int __init ds18b20_init(void)
{
	int ret;
	ret = misc_register(&misc);
	
	if(ret!=0)
	{
		printk("init led unsuccessfully!\n");
		return -1;
	}

	return 0;
}

static void __exit ds18b20_exit(void)
{

	misc_deregister(&misc);
	printk("led  exit!\n");
} 



module_init(ds18b20_init);
module_exit(ds18b20_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("www.waveshare.net");
MODULE_DESCRIPTION("module Driver");
