#include <linux/module.h>

#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/mutex.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/stat.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/tty.h>
#include <linux/kmod.h>
#include <linux/gfp.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/pm.h>
#include <linux/slab.h>
#include <linux/sysctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <linux/poll.h> 
#define IMX_GPIO_NR(bank, nr)		(((bank) - 1) * 32 + (nr))

#define DEVICE_NAME 	"led_show"
#define _debug(fmt, arg...)	printk(KERN_ERR fmt, ##arg)


#define LED_ON 0X00
#define LED_OFF 0X01


typedef struct __GPIO_DESC {
	int gpio;
	const char *desc;
}led_desc;

static led_desc led_dev_data[] = {
	{IMX_GPIO_NR(6,11),"led1"},		
	{IMX_GPIO_NR(6,8),"led2"},		
	{IMX_GPIO_NR(6,7),"led3"},		
	{IMX_GPIO_NR(6,9),"led4"},		
			
};

static long led_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
	
	if(cmd != LED_ON && cmd != LED_OFF)
	{
		return -1;
	}
	
	gpio_set_value(led_dev_data[arg].gpio, cmd);
	return 0;
}

static int led_open(struct inode *inode, struct file *file)
{
	
	int ret = -1;
	
	int i;
	for( i = 0 ;i < 4; i++)
	{
		ret = gpio_request(led_dev_data[i].gpio,led_dev_data[i].desc);
		if(ret < 0)
		{
			_debug("request pin %d failed!\n",i);
			
		}
		gpio_direction_output(led_dev_data[i].gpio, 0);		
	}

	
	// gpio_set_value(led_dev_data[0].gpio, 1);			

	return 0;

}

static int led_close(struct inode *inode, struct file *file)
{
	int i = 0;
	
	for(i=0 ; i<sizeof(led_dev_data)/sizeof(led_desc) ; i++)
	{
		gpio_free(led_dev_data[i].gpio);
	}
	return 0;
}


static struct file_operations led_fops=
{
    .owner			=	THIS_MODULE,
    .unlocked_ioctl	=	led_ioctl,
	.open 			= 	led_open,
	.release	 	= 	led_close,
};

static struct miscdevice misc = {
	.minor	= MISC_DYNAMIC_MINOR,
	.name	= DEVICE_NAME,
	.fops	= &led_fops,
};


static int __init led_t_init(void)
{
	int ret;
	ret = misc_register(&misc);
	
	if(ret!=0)
	{
		return -1;
	}
	printk("Register led successfully!\n");	
	return 0;
}

static void __exit led_t_exit(void)
{

	misc_deregister(&misc);
	printk("spi_io driver exit!\n");
} 



module_init(led_t_init);
module_exit(led_t_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("WS<www.waveshare.net>");
MODULE_DESCRIPTION("module Driver");
