#include <linux/module.h>

#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/kernel.h>
#include <linux/major.h>
#include <linux/mutex.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/stat.h>
#include <linux/init.h>
#include <linux/device.h>
#include <linux/tty.h>
#include <linux/kmod.h>
#include <linux/gfp.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/pm.h>
#include <linux/slab.h>
#include <linux/sysctl.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <linux/poll.h> 
#define IMX_GPIO_NR(bank, nr)		(((bank) - 1) * 32 + (nr))

#define DEVICE_NAME 	"spi_io"
#define _debug(fmt, arg...)	printk(KERN_ERR fmt, ##arg)


#define PULLUP 0X01
#define PULLDOWN 0X00


typedef struct __GPIO_DESC {
	int gpio;
	const char *desc;
}led_desc;

static led_desc led_dev_data[] = {
	{IMX_GPIO_NR(2,26),"spi_io"},		
};

static long led_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{

	
	//printk(KERN_ERR "%s called,cmd = 0x%x,arg =0x%x,\n",__func__,cmd,arg);
	if(cmd != PULLUP && cmd != PULLDOWN)
	{
		//printk("ioctl cmd erro!\n");
		return -1;
	}

	gpio_set_value(led_dev_data[0].gpio, cmd);
	return 0;
}

static int led_open(struct inode *inode, struct file *file)
{
	
	int ret = -1;
	
	ret = gpio_request(led_dev_data[0].gpio,led_dev_data[0].desc);
	if(ret < 0)
	{
		
		return ret;
	}
	gpio_direction_output(led_dev_data[0].gpio, 0);		/* <>set gpio's direction*/
	gpio_set_value(led_dev_data[0].gpio, 1);			/* <>set gpio output high level*/

	return 0;

}

static int led_close(struct inode *inode, struct file *file)
{
	int i = 0;
	
	for(i=0 ; i<sizeof(led_dev_data)/sizeof(led_desc) ; i++)
	{
		gpio_free(led_dev_data[i].gpio);
	}
	return 0;
}


static struct file_operations led_fops=
{
    .owner			=	THIS_MODULE,
    .unlocked_ioctl	=	led_ioctl,
	.open 			= 	led_open,
	.release	 	= 	led_close,
};

static struct miscdevice misc = {
	.minor	= MISC_DYNAMIC_MINOR,
	.name	= DEVICE_NAME,
	.fops	= &led_fops,
};


static int __init led_t_init(void)
{
	int ret;
	ret = misc_register(&misc);
	
	if(ret!=0)
	{
		return -1;
	}
	printk("Register led successfully!\n");	
	return 0;
}

static void __exit led_t_exit(void)
{

	misc_deregister(&misc);
	printk("spi_io driver exit!\n");
} 



module_init(led_t_init);
module_exit(led_t_exit);
MODULE_LICENSE("GPL");
MODULE_AUTHOR("WS<www.waveshare.net>");
MODULE_DESCRIPTION("module Driver");
